import { IRange } from "../interfaces/IRange";
import { ModbusFunctionCode } from "./ModbusFunctionCode";

const READ_REGISTER_COUNT_RANGE: IRange = {min: 0x0001, max: 0x007F};
const WRITE_REGISTER_COUNT_RANGE: IRange = {min: 0x0001, max: 0x007B};
const COIL_VALUE_RANGE: IRange = {min: 0, max: 1};

const REGISTER_FUNCTION_CODES: Array<ModbusFunctionCode> = [
  ModbusFunctionCode.FC3,
  ModbusFunctionCode.FC4,
  ModbusFunctionCode.FC6,
  ModbusFunctionCode.FC16,
];

const COIL_FUNCTION_CODES: Array<ModbusFunctionCode> = [
  ModbusFunctionCode.FC1,
  ModbusFunctionCode.FC2,
  ModbusFunctionCode.FC5,
  ModbusFunctionCode.FC15,
];

export const MODBUS = {
  COIL_VALUE_RANGE,
  READ_REGISTER_COUNT_RANGE,
  WRITE_REGISTER_COUNT_RANGE,
  REGISTER_FUNCTION_CODES,
  COIL_FUNCTION_CODES,

  PORT:        802,
  PROTOCOL_ID: 0,

  MAX_TCP_TRANSACTIONS:     0xFFFF,
  MAX_REGISTER_PER_REQUEST: 125,

  MAX_COILS_PER_REQUEST: 2000,

  // device addresses
  ADDRESS: {
    MIN:       1,
    MAX:       247,
    BROADCAST: 0,
  },

  OFFSET: {
    REQUEST:        {
      TRANSACTION_ID: 0,
      PROTOCOL_ID:    2,
      MSG_LENGTH:     4,
      MODBUS_ID:      6,
      FUNCTION_CODE:  7,
      REG_ADDRESS:    8,
      REG_COUNT:      10,
    },
    RESPONSE:       {
      TRANSACTION_ID: 0,
      PROTOCOL_ID:    2,
      MSG_LENGTH:     4,
      MODBUS_ID:      6,
      FUNCTION_CODE:  7,
      DATA_LENGTH:    8,
      DATA:           9,
      EXCEPTION:      8,
    },
    WRITE_SINGLE:   {
      TRANSACTION_ID: 0,
      PROTOCOL_ID:    2,
      MSG_LENGTH:     4,
      MODBUS_ID:      6,
      FUNCTION_CODE:  7,
      REG_ADDRESS:    8,
      DATA:           10,
    },
    WRITE_MULTIPLE: {
      TRANSACTION_ID: 0,
      PROTOCOL_ID:    2,
      MSG_LENGTH:     4,
      MODBUS_ID:      6,
      FUNCTION_CODE:  7,
      REG_ADDRESS:    8,
      REG_COUNT:      10,
      BYTE_COUNT:     12,
      DATA:           13,
    },

  },

  NAN:                      0xFFFF,
  WRITE_COIL_ENABLE_VALUE:  0xFF00,
  WRITE_COIL_DISABLE_VALUE: 0x0000,

  REGISTER_RANGE: {
    min: 0,
    max: 0xFFFF,
  },

  LENGTH: {
    MBAP_HEADER:          7,
    MODBUS_ID:            1,
    FUNCTION_CODE:        1,
    DATA_LENGTH:          1,
    CHECKSUM:             2,
    REQUEST_MESSAGE:      6,

    RESPONSE: {
      READ_WITHOUT_CONTENT:         9,
      READ_WITH_EXCEPTION:          3,
      WRITE_SINGLE_WITHOUT_CONTENT: 10,
      WRITE_MULTIPLE:               12,
    },

    MINIMAL_PDU:               1,
    READ_REGISTER_REQUEST_PDU: 5,
    WRITE_SINGLE_COIL_PDU:     5,
    WRITE_SINGLE_REGISTER_PDU: 5,
    WRITE_MULTIPLE_REGISTERS_REQUEST_PDU_WITHOUT_PAYLOAD: 1 + 2 + 2 + 1,
    WRITE_MULTIPLE_REGISTERS_RESPONSE_PDU:                5,

    EXCEPTION:     9,
  },

  BYTES_PER_REGISTER: 2,
  BIT_PER_BYTE:       8,


  READ_COIL_FUNCTION_CODES: [ModbusFunctionCode.FC1, ModbusFunctionCode.FC2],

  EXCEPTION_FUNCTION_CODE_OFFSET: 0x80,
};