import { IModbusRequest } from "../interfaces/IModbusRequest";
import * as tls from "tls";
import { ResponseHelper } from "../helper/ResponseHelper";
import { ModbusException } from "../constants/ModbusException";
import { IModbusWriteRequest } from "../interfaces/IModbusWriteRequest";

export class ExceptionService {
  public static sendIllegalFunctionException(request: IModbusRequest | IModbusWriteRequest, socket: tls.TLSSocket, message?: string) {
    this.sendException(ModbusException.IllegalFunction, request, socket, message)
  }

  public static sendIllegalDataAddressException(request: IModbusRequest | IModbusWriteRequest, socket: tls.TLSSocket, message?: string) {
    this.sendException(ModbusException.IllegalDataAddress, request, socket, message)
  }
  public static sendIllegalDataValueException(request: IModbusRequest | IModbusWriteRequest, socket: tls.TLSSocket, message?: string) {
    this.sendException(ModbusException.IllegalDataValue, request, socket, message)
  }

  public static sendSlaveDeviceFailureException(request: IModbusRequest | IModbusWriteRequest, socket: tls.TLSSocket, message?: string) {
    this.sendException(ModbusException.SlaveDeviceFailure, request, socket, message)
  }

  private static sendException(exception: ModbusException, request: IModbusRequest | IModbusWriteRequest, socket: tls.TLSSocket, message?: string) {
    if (message !== undefined) {
      console.log(message);
    }

    const exceptionResponse = ResponseHelper.buildExceptionResponseBuffer(request, exception);

    socket.write(exceptionResponse);
  }
}